#include <windows.h>
#include <stdio.h>
#include <time.h>
#include "pthread.h"
/*
   The header file defines a particular kind of buffer data
   structure and it defines the two functions putInBuffer()
   (used by the producer) and TakeOutBuffer() (used by the
   consumer) needed for manipulating its kind of buffer.
*/
#include "StackBuffer.h"

#define BUFFER_COUNT 3
#define P_THREAD_COUNT 3
#define C_THREAD_COUNT 3
#define THREAD_COUNT P_THREAD_COUNT + C_THREAD_COUNT


// Global variables
Buffer buffers[BUFFER_COUNT];   // array of Buffer data structures
// You need to declare arrays of mutexes and condition variables
// to bound the buffers. Each buffer needs one mutex and two
// condition variables.


static int runFlag = TRUE;


int main( int argc, char *argv[])
{
   // Producer and consumer thread function prototypes.
   void *producer(void*);
   void *consumer(void*);

   pthread_t p_tid[P_THREAD_COUNT];    // thread ids
   pthread_t c_tid[C_THREAD_COUNT];
   int producerParam[P_THREAD_COUNT];  // thread function parameters
   int consumerParam[C_THREAD_COUNT];

   int i;

   // Call the function that initializes all of the buffers.
   initializeBuffers(buffers, BUFFER_COUNT);


   // Create the arrays of mutexes and condition variables
   // needed to bound the buffers so that the buffers do
   // not overflow or underflow with data. The buffers are
   // initially empty.

   // Start the consumer threads. Pass the i'th consumer thread the
   // thread parameter 'A'+i (so the consumer threads have thread
   // parameters 'A', 'B', 'C', etc.). Be sure to check that the
   // consumer threads were created without any error.

   // Start the producer threads. Pass the i'th producer thread the
   // thread parameter '1'+i (so the producer threads have thread
   // parameters '1', '2', '3', etc.). Be sure to check that the
   // producer threads were created without any error.


   Sleep(5000);     // let the threads run for a while
   runFlag = FALSE; // now stop the threads
   for (i = 0; i < P_THREAD_COUNT; i++)
   {  pthread_join(p_tid[i], NULL); }
   for (i = 0; i < C_THREAD_COUNT; i++)
   {  pthread_join(c_tid[i], NULL); }
   printf("\nThreads stopped.\n");
   pthread_exit(NULL);
   return 0;
}//main()


// Put the thread parameter in a buffer (the producer).
void *producer(void *threadParam)
{
   int bufferNo;

   printf("Started producer thread %c\n", *(char*)threadParam);

   // seed the random number generator
   srand( time(NULL) + *(int*)threadParam );

   while(runFlag) // put the thread parameter in a buffer
   {
      // Pick a random buffer number between 0 and BUFFER_COUNT-1.
      bufferNo = rand()%BUFFER_COUNT;


     // Below is the call to putInBuffer() that puts an item (the
     // thread parameter) into the buffer. You need to keep the
     // producers from overflowing the buffer and you need to signal
     // a consumer when there's an item in the buffer.


      // begin critical section
      putInBuffer(*(char*)threadParam, &(buffers[bufferNo]));
      // Print the buffer.
      printBuffer(&(buffers[bufferNo]), bufferNo);
      // end of critical section



      Sleep(10*(rand()%10));  // sleep a random amount
   }
   return 0;
}//producer()


// Remove an item from a buffer (the consumer).
void *consumer(void *threadParam)
{
   int bufferNo;

   printf("Started consumer thread %c\n", *(char*)threadParam);

   // seed the random number generator
   srand( time(NULL) + *(int*)threadParam );

   while(runFlag)  // remove an item from a buffer
   {
      // Pick a random buffer number between 0 and BUFFER_COUNT-1.
      bufferNo = rand()%BUFFER_COUNT;


     // Below is the call to takeFromBuffer() that takes an item out
     // of the buffer. You need to keep the consumers from underflowing
     // the buffer and you need to signal a producer when an item is
     // removed from the buffer.


      // begin critical section
      takeFromBuffer(*(char*)threadParam, &(buffers[bufferNo]));
      // Print the buffer.
      printBuffer(&(buffers[bufferNo]), bufferNo);
      // end of critical section



      Sleep(10*(rand()%10));  // sleep a random amount
   }
   return 0;
}//consumer()