#include <stdio.h>

// Define how many items the stack can hold.
#define SIZE 11

// Here is the data structure that defines a stack type buffer.
// The array has SIZE+2 entries so that it can hold SIZE pieces
// of data, a special character to denote the end of the stack
// space, and a null character ('\0') to denote the end of a string
// (the buffer will be printed out as one whole string).
typedef struct
{
   char buffer[SIZE+2];
   unsigned int top;   // top of stack index
} Buffer;


// Prototype for the printBuffer function (defined below).
void printBuffer(Buffer* buffer_p, int bufferNo);


// Determine if the stack data structure is empty.
int bufferIsEmpty(Buffer* buffer_p)
{
   return ( 0 == buffer_p->top );
}//bufferIsEmpty()


// Determine if the stack data structure is full.
int bufferIsFull(Buffer* buffer_p)
{
   return ( SIZE == buffer_p->top );
}//bufferIsFull()


// Function for pushing an item onto the stack data structure.
void putInBuffer(char item, Buffer* buffer_p)
{
   // Push an item on the stack and adjust the top index.
   (buffer_p->buffer)[buffer_p->top] = item;
   (buffer_p->top)++;
}//putInBuffer()


// Function for popping an item off of the stack data structure.
// (Replace the top of stack with parameter item.)
char takeFromBuffer(char item, Buffer* buffer_p)
{
   char result;

   // Remove the item from the top of the stack (and store that
   // item in result), then adjust the top of stack index. Also,
   // either put the character '0' into the buffer where the top
   // of stack item used to be, so that the stack looks better
   // when it is printed out, OR put the parameter "item" in place
   // of the top of stack. This helps to see who called this function.
   (buffer_p->top)--;
   result = (buffer_p->buffer)[buffer_p->top];
 //(buffer_p->buffer)[buffer_p->top] = '0';  // mark space as empty
   (buffer_p->buffer)[buffer_p->top] = item; // mark space as empty
   return result;
}//takeFromBuffer()


// Function that prints a stack buffer in its own column in the output.
void printBuffer(Buffer* buffer_p, int bufferNo)
{
   // a string of 70 spaces
   char* spaces = "                                                                      ";

   // print the stack and show the top of stack index
   printf("%.*s%s\n%.*s%.*s^\n", bufferNo*(SIZE+2), spaces, buffer_p->buffer,
                                 bufferNo*(SIZE+2), spaces, buffer_p->top, spaces);
}//printBuffer()


// Function that initializes all of the buffers.
void initializeBuffers(Buffer* buffers, int bufferCount)
{
   int i, j;

   // initialize the stack buffers
   for (i = 0; i < bufferCount; i++)
   {
      for (j = 0; j < SIZE; j++)
      {
        (buffers[i].buffer)[j] = '0';     // initialize each buffer to be empty
      }
      (buffers[i].buffer)[SIZE] = '*';    // mark the end of each stack with a special character
      (buffers[i].buffer)[SIZE+1] = '\0'; // null terminate each of the buffers
      buffers[i].top = 0;                 // initialize each stack buffer index
   }

   printf("Here is the initial state of the buffers.\n\n");
   // print column headings (each column is SIZE+2 characters wide)
   for (i = 0; i < bufferCount; i++)
   {
      printf("  buffer[%d]%.*s", i, (SIZE+2)-11, "          ");
   }
   printf("\n\n");
   for (i = 0; i < bufferCount; i++)  // print each buffer
   {
      printBuffer(&(buffers[i]), i);
   }

}//initializeBuffers()